      subroutine pdlev (Nx,Mtx,mex,xl,xu,Ax,bx,xin,xout,izs,rzs,
     *                  dzs,fun,ipin,rpin,ipout,rpout,war,iwar)
      implicit integer *4 (i-n)
      implicit real *8 (a-h,o-z)
      external fun
      real *4 rzs
      dimension xl(*),xu(*),Ax(*),bx(*),xin(*),xout(*),
     1          ipin(*),ipout(*),rpin(*),rpout(*),war(*),
     2          iwar(*),izs(*),rzs(*),dzs(*)
      common /cmache/ epsp /powit/ itrpw
      character *40 txt
C--------------------------------------------------------------
C     Solver for the problem
C          min f(x)
C          subject to 
C          Ax[i]x + bx[i]  = 0, i = 1,...,mex,
C          Ax[i]x + bx[i] >= 0, i = mex+1,...,Mtx,
C          xl <= x <= xu, x={x[j]}, j = 1,...,Nx.
C--------------------------------------------------------------
C-----Semantics of formal parameters Input/Output:
C     war(1:nwar)     real working array
C     iwar(1:niwar)   integer working array
C-----INPUT:
C     Nx            dimension of x
C     Mtx           see formulation of the problem
C     mex           see formulation of the problem
C     xl(1:Nx)      see formulation of the problem
C     xu(1:Nx)      see formulation of the problem
C     Ax(1:Mtx*Nx)  matrix of linear constraints represented
C                   as a linear array; Ax[i] is the fragment
C                   Ax(1+(i-1)*Nx:i*Nx) of the array
C     bx(1:Mtx)     see formulation of the problem
C     izs           integer data array for procedure fun
C     rzs           real *4 data array for procedure fun
C     dzs           real *8 data array for procedure fun
C     fun           subroutine
C                      fun (indic,x,Fxy,gx,izs,rzs,dzs)
C                   computing the value (Fxy) and the gradient
C                   (gx) of the objective or of the measure
C                   function at given point (x)
C                   ! indic must always be equal 0 for consistent
C                   and 1 for inconsistent problems
C     xin(1:Nx)     initial approximate solution
C     ipin(1:8)     integer control parameters
C                 ipin(1) # of iterations allowed <limm=200>
C                 ipin(2) printlevel <iprnt=-1>
C                      -2 no protocol file
C                      -1 no screen messages
C                       0 screen information after termination
C                       1 run-time screen information
C                 ipin(3) # of channel for protocol file <io1=15>
C                 ipin(4) <nwar>
C                 ipin(5) <niwar>
C                 ipin(6) # of channel of resulting file <io2=16>
C                 ipin(7:8) not used
C     rpin(1:8)     real control parameters
C                 rpin(1) relative accuracy <epsrel=1.D-6>
C                 rpin(2) bundle relaxing coefficient
C                         <cfrelb=0.5D0>
C                 rpin(3) tolerance for constraints <tol=1.D-6>
C                 rpin(4) square root of the diag. entry
C                         in zero matrix <epsd=1.D-4>
C                 rpin(5) zero for Powell <epsp=1.D-8>
C                 rpin(6) level <qlv=0.5D0>
C                 rpin(7:8) not used
C-----OUTPUT:
C     xout(1:Nx)    resulting approximate solution
C     ipout(1:8)    integer diagnostics
C                 ipout(1)   reason for termination
C                        -4  errors in subroutine fun
C                        -3  shrinking bundle leads to loop
C                        -2  not enough real memory
C                        -1  not enough integer memory
C                         0  normal termination
C                        >0  cannot compute something
C                 ipout(2)   what is found
C                        -2  nothing found
C                        -1  unfeasibility found
C                         0  feasible solution found
C                         1  optimal solution found
C                         2  auxiliary results
C                 ipout(3)   # of iterations <itr>
C                 ipout(4)   # of feasible steps <indic0+1>
C                 ipout(5)   icodi
C                 ipout(6)   max value of bundle <MaxBnd>
C                 ipout(7)   # of bundle shrinkings <Nbndl>
C                 ipout(8)   # of auxilyary problems solved <ncrit>
C                 ipout(9)   # of Oracle iterations <isumor>
C                 ipout(10)  # of LP iterations <isumpl>
C                 ipout(11)  # of QP iterations <isumpq>
C                 ipout(12)  # of auxiliary iterations <isumpc>
C                 ipout(13:..) not used
C     rpout(1:8)    real diagnostics
C                 rpout(1)   upper bound for absolute accuracy
C                            <Gap=Fmod>
C                 rpout(2)   Fmodx
C                 rpout(3)   Fmody
C                 rpout(4)   relative accuracy <Relacc>
C                 rpout(5)   Fmax
C                 rpout(6)   Fmin
C                 rpout(7)   Fxy
C                 rpout(8)   Avit
C                 rpout(9:...) not used
C=====DECLARATIONS=============================================
      Zero=0.0D0
      Half=0.5D0
      One=1.0D0
      Two=2.0D0
      Ten=1.0D1
      Hund=1.0D2
      W20=1.0D20
      dgr=7.7777777D77
      Relacc=dgr
C=====LOCAL CONTROL PARAMETERS=================================
C     without debugging (idebl=0) / using debugging (idebl=1)
      idebl=0
C     no auxiliary results (ifreq=0) / after <ifreq> iterations
      ifreq=100
C     no shrinking bundle (ishrb=0) / after at most <ishrb*Nx> it.
      ishrb=10
C     solving auxiliary problem after <icrit>0> iterations
      icrit=10
C=====GLOBAL CONTROL PARAMETERS================================
C     (limm) max number of iterations
      if (ipin(1).le.0) ipin(1)=200
      limm=ipin(1)
C     (iprnt) screen messages and protocol file
      if (ipin(2).lt.-2) ipin(2)=-3
      iprnt=ipin(2)
C     (io1) # of output channel for Solver
      if (ipin(3).le.0) ipin(3)=15
      io1=ipin(3)
C     (nwar) length of real working array war
      if (ipin(4).le.0) then
         ipout(1)=-2
         ipout(2)=-2
         goto 9999
      endif
      nwar=ipin(4)
C     (niwar) length of integer working array iwar
      if (ipin(5).le.0) then
         ipout(1)=-1
         ipout(2)=-2
         goto 9999
      endif
      niwar=ipin(5)
C-----not used
      ipin(6)=0
      ipin(7)=0
      ipin(8)=0
C-----
C     (epsrel) last relative accuracy required-----------------
      if (rpin(1).le.Zero) rpin(1)=1.0D-6
      epsrel=rpin(1)
C     (cfrelb) coefficient for shrinking bundle
      if (rpin(2).lt.Zero) rpin(2)=Half
      if (rpin(2).gt.One) rpin(2)=One
      cfrelb=rpin(2)
C     (tol) tolerance
      if (rpin(3).le.Zero) rpin(3)=1.0D-6
      tol=rpin(3)
C     (epsd) squared root of the diaganal in Zero matrix
      if (rpin(4).le.Zero) rpin(4)=1.0D-4
      epsd=rpin(4)
      epsd2=epsd*epsd
C     (epsp) Zero for Powell
      if (rpin(5).le.Zero) rpin(5)=1.0D-8
      epsp=rpin(5)
C     (qlv) level
      if (rpin(6).le.Zero) rpin(6)=Half
      if (rpin(6).ge.One) rpin(6)=Half
      qlv=rpin(6)
C     not used
      rpin(7)=Zero
      rpin(8)=Zero
C=====REAL AND INTEGER CONTROL DIAGNOSTICS=====================
      ipout(1)=-789
      ipout(2)=-2
      do i=1,18
         ipout(i+2)=0
      enddo
      do i=1,18
         rpout(i)=dgr
      enddo
C-----
      dinf=1234567.D18
      Erri=W20
      Fmod=W20
      Fmodc=W20
      Fmodd=W20
      Fmax=-dinf
      Fmin=dinf
      Fest=-dinf
      Fmodx=-dinf
      Fmody=-Fmin
      Relshr=W20
      Gapsh=W20
      Mxbndl=0
      Nmbndl=0
      Mav=0
      if (ishrb.ne.0) then
         MaxBnd=ishrb*Nx
         if (Maxbnd.eq.0) Maxbnd=1
      endif
C=====MEMORY DISTRIBUTION======================================
C-----Sizes
      Ny=0
      Nx0=Nx+1
      Nx1=Nx+2
      nmx=Nx1
      mmx=Mtx
      mmrq=Nx1*(Mtx+9)+3*Mtx+5
      mmcf=Nx1+4
      mprr=3*nmx*nmx/2+10*nmx+2*mmx+1
      mprq=mprr+nmx*nmx+5*nmx+3*mmx+4
      mrq=mmrq+mprq
      mcf=mmcf
C-----Constant part
      nxxc=0
      nxxl=nxxc+Nx1
      nxxq=nxxl+Nx1
      nxl=nxxq+Nx1
      nxu=nxl+Nx1
      ntrx=nxu+Nx1
      nbdx=ntrx+Nx1
C-----Variable part
      mpww=0
      lpir=nmx
      if (mpww+lpir.gt.niwar) then
         ipout(1)=-1
         ipout(2)=-2
         ipout(3)=0
         goto 9999
      endif
      lim=(nwar-mrq)/mcf
      if (ishrb.ne.0) then
         if (lim.lt.MaxBnd) then
            ipout(1)=-2
            ipout(2)=-2
            ipout(3)=0
            goto 9999
         endif
      endif
      nxbt=nbdx+Nx1*(Mtx+lim+1)
      nxbs=nxbt+(Mtx+lim+1)
      nxdl=nxbs+(Mtx+lim+1)
      npwl=nxdl+(Mtx+lim+1)+2*Nx1
      npwc=npwl
      npwd=npwc+nmx*nmx
      npwu=npwd+nmx
      npww=npwu+(mmx+lim+1)+2*nmx
      lprr=nwar-npww
      nrab=mprr
      if (lprr.lt.nrab) then
         ipout(1)=-2
         ipout(2)=-2
         ipout(3)=0
         goto 9999
      endif
C-----Initial part---------------------------------------------
      Diamx=Zero
      if (Nx.ne.0) then
         do i=1,Nx
            ww=xu(i)-xl(i)
            Diamx=Diamx+ww*ww
         enddo
         Diamx=dsqrt(Diamx)
      endif
C-----
      if (Mtx.ne.0) then
         nt=nbdx
         is=0
         do i=1,Mtx
            war(nxbt+i)=bx(i)
            war(nxbs+i)=bx(i)
            do j=1,Nx
               nt=nt+1
               is=is+1
               war(nt)=Ax(is)
            enddo
            nt=nt+1
            war(nt)=Zero
            nt=nt+1
            war(nt)=Zero
         enddo
      endif
      if (Nx.ne.0) then
         do i=1,Nx
            war(nxl+i)=xl(i)
            war(nxu+i)=xu(i)
         enddo
      endif
      war(nxl+Nx0)=Zero
      war(nxu+Nx0)=dinf
      war(nxl+Nx1)=-dinf
      war(nxu+Nx1)=dinf
C-----Check whether initial x is in box------------------------
      viol=Zero
      if (Nx.ne.0) then
         do i=1,Nx
            ww=xin(i)
            wl=xl(i)
            wu=xu(i)
            s1=ww-wl
            s2=wu-ww
            if (s1.lt.Zero.or.s2.lt.Zero) then
               if (Mtx.gt.0) then
                  viol=tol+One
                  goto 33
               else
                  ww=Half*(wl+wu)
                  war(nxxc+i)=ww
               endif
            endif
         enddo
      endif
      if (Mtx.eq.0) goto 33
      is=0
      do i=1,Mtx
         sbx=bx(i)
         s=sbx
         do j=1,Nx
            is=is+1
            s=s+Ax(is)*xin(j)
         enddo
         if (i.le.mex) then
            s=dabs(s)
         else
            if (s.lt.Zero) then
               s=-s
            else
               s=Zero
            endif
         endif
         s=s/(One+dabs(sbx))
         if (s.gt.viol) viol=s
         if (viol.gt.tol) goto 33
      enddo
C-----If x not feasible, perform QP(x0)------------------------
   33 if (Nx.ne.0) then
         do i=1,Nx
            war(nxxc+i)=xin(i)
         enddo
      endif
      war(nxxc+Nx1)=Zero
      if (viol.le.tol) goto 38
      txt='Initial x is unfeasible'
      call msg (ipin,-1,1,txt,0,Zero)
      mtt=Mtx
      me=mex
      mmax=mtt
      nmax=Nx1
      mnn=mtt+2*Nx1
      nt=npwc
      do i=1,Nx1
         war(npwd+i)=-war(nxxc+i)
         do j=1,Nx1
            nt=nt+1
            ww=Zero
            if (i.eq.j) ww=One
            war(nt)=ww
         enddo
      enddo
      war(nxl+Nx1)=-One
      war(nxu+Nx1)=One
      iwar(mpww+1)=0
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' Before QP(x0)'
         write (io1,*) ' d='
         write (io1,*) (war(npwd+i),i=1,Nx1)
         write (io1,*) ' c='
         write (io1,*) (war(npwc+i),i=1,Nx1*Nx1)
         write (io1,*) ' b='
         write (io1,*) (war(nxbt+i),i=1,mtt)
         write (io1,*) ' a='
         write (io1,*) (war(nbdx+i),i=1,Nx1*mtt)
         write (io1,*) ' xl='
         write (io1,*) (war(nxl+i),i=1,Nx1)
         write (io1,*) ' xu='
         write (io1,*) (war(nxu+i),i=1,Nx1)
      endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      call ql0001 (mtt,me,mmax,Nx1,nmax,mnn,
     1             war(npwc+1),war(npwd+1),war(nbdx+1),
     2             war(nxbt+1),war(nxl+1),war(nxu+1),
     3             war(nxxc+1),war(npwu+1),iout,ifail,
     4             iprnt,war(npww+1),lprr,iwar(mpww+1),lpir)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if (ifail.gt.2) then
         ipout(1)=ifail+100
         ipout(2)=-2
         txt=' QP(x0) : Failure'
         call msg (ipin,-1,1,txt,0,Zero)
         goto 9999
      endif
   38  do i=1,Nx1
         war(nxxl+i)=war(nxxc+i)
      enddo
C=====COUNTERS=================================================
   50 itr=0
      itrc=0
      ncrit=0
      itrp=0
      m=0
      nbdxc=nbdx+Mtx*Nx1
      isumor=0
      isumpl=0
      isumpq=0
      isumpc=0
      indic0=-1
C=====MAIN LOOP================================================
  100 continue
      itr=itr+1
      m=m+1
      txt=' '
      call msg (ipin,-1,1,txt,0,Zero)
      txt='ITERATION # '
      call msg (ipin,0,1,txt,itr,Zero)
      if (itr.gt.limm.or.m.gt.lim) then
         if (itr.gt.limm) ipout(1)=-10
         if (m.gt.lim) ipout(1)=-11
         ipout(2)=0
         itr=itr-1
         goto 9999
      endif
C=====ORACLE===================================================
 110  continue
C-----itr => izs(6)
      izs(6)=itr
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' Before FUN ============, Fxy=',Fxy
         write (io1,*) ' xc='
         write (io1,*) (war(nxxc+i),i=1,Nx1)
         write (io1,*) ' resx='
         write (io1,*) (war(nbdxc+i),i=1,Nx1)
      endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      call fun (indic,war(nxxc+1),Fxy,war(nbdxc+1),izs,rzs,dzs)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' After FUN ============, Fxy=',Fxy
         write (io1,*) ' indic=',indic
         write (io1,*) ' xc='
         write (io1,*) (war(nxxc+i),i=1,Nx1)
         write (io1,*) ' resx='
         write (io1,*) (war(nbdxc+i),i=1,Nx1)
      endif
C--------------------------------------------------------------
C     indic=0   <= OK
C     indic=1   <= infeasibility
C     indic=2   <= errors from Solpwl
C     otherwise bad
C--------------------------------------------------------------
      if (indic.lt.0.or.indic.gt.1) then
         ipout(1)=-4
         ipout(2)=-indic
         ipout(3)=itr
         txt='Errors from subroutine fun'
         call msg (ipin,-1,0,txt,0,Zero)
         goto 5555
      endif
      itor=1
C-----izs(7) => kki
      kki=izs(7)
      isumor=isumor+kki
C=====END OF ORACLE, NEW RECORD================================
      if (indic.eq.0) then
         indic0=indic0+1
         if (Fmax.lt.Fxy) Fmax=Fxy
         if (Fmin.gt.Fxy) Fmin=Fxy
         imody=0
         if (Fmody.lt.-Fmin) then
            Fmody=-Fmin
            imody=1
         endif
      else
         imody=0
      endif
C-----x
      s=-Fxy
      Fnorm=Zero
      if (Nx.ne.0) then
         do i=1,Nx
            wwc=war(nxxc+i)
            j=nbdxc+i
            wwb=war(j)
            if (imody.ne.0) war(ntrx+i)=wwc
            s=s+wwc*wwb
            war(j)=-wwb
            Fnorm=Fnorm+wwb*wwb
         enddo
         Fnorm=dsqrt(Fnorm)
      endif
      ww0=-Fnorm
      war(nbdxc+Nx0)=ww0
      ww1=Zero
      if (indic.eq.0) ww1=One
      war(nbdxc+Nx1)=ww1
      ww=Zero
      if (indic0.ge.0) ww=-Fmin
      if (imody.ne.0) then
         war(ntrx+Nx0)=Fxy
         war(ntrx+Nx1)=ww
      endif
      nbdxc=nbdxc+Nx1
      war(nxbt+Mtx+m)=s
      if (indic.eq.0) s=s+Fxy
      war(nxbs+Mtx+m)=s
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) ' indic,Fnorm=',indic,Fnorm
         write (io1,*)
         write (io1,*) ' xbt='
         write (io1,*) (war(nxbt+i),i=1,Mtx+m)
         write (io1,*)
         write (io1,*) ' xbs='
         write (io1,*) (war(nxbs+i),i=1,Mtx+m)
         write (io1,*)
         write (io1,*) ' pointx='
         write (io1,*) (war(ntrx+i-Nx1),i=1,Nx1)
         write (io1,*)
         write (io1,*) ' matx='
         write (io1,*) (war(nbdxc+i-Nx1),i=1,Nx1)
      endif
C-----y
      txt=' Function     <='
      call msg (ipin,1,1,txt,0,Fxy)
      if (Fmodx.lt.Fest) then
         ipout(1)=0
         ipout(2)=-1
         ipout(3)=itr
         goto 9999
      endif
C=====Checking whether minimum (x) has been varied=============
  200 continue
      nxb=nxbt
      if (itr.ne.1) then
         s=war(nxb+Mtx+m)
         ix=nbdxc-Nx1
         if (Nx.ne.0) then
            do i=1,Nx
               ix=ix+1
               s=s+war(ix)*war(nxxl+i)
            enddo
         endif
         ix=ix+1
         s=s+war(ix)*war(nxxl+Nx0)
         ix=ix+1
         s=s-war(ix)*Fmody
         if (s.lt.Zero) goto 210
         if (imody.ne.0) goto 210
         itilx=0
         war(nxdl+Mtx+m)=Zero
         wwrt=Fmod
         goto 220
      endif
C=====Computing minimum of the model LP(x)=====================
  210 continue
      if (Nx.ne.0) then
         do i=1,Nx
            war(npwd+i)=-epsd2*war(nxxc+i)
         enddo
      endif
      ww0=-One
      if (itr.ne.1) ww0=-One-epsd2*Fmod
      war(npwd+Nx0)=ww0
      ww1=Zero
      if (indic0.ge.0) ww1=epsd2*Fmody
      war(npwd+Nx1)=ww1
      nt=npwc
      do i=1,Nx1
         do j=1,Nx1
            nt=nt+1
            ww=Zero
            if (i.eq.j) ww=epsd
            war(nt)=ww
         enddo
      enddo
      mtt=Mtx+m
      me=mex
      mmax=mtt
      nmax=Nx1
      mnn=mtt+2*Nx1
      ww0=dinf
      if (itr.ne.1) ww0=Fmod
      war(nxl+Nx0)=Zero
      war(nxu+Nx0)=ww0
      ww1=Zero
      if (indic0.ge.0) ww1=-Fmody
      war(nxl+Nx1)=ww1
      war(nxu+Nx1)=ww1
      iwar(mpww+1)=0
      itilx=0
C-----
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' Before LP(x)'
         write (io1,*) ' d='
         write (io1,*) (war(npwd+i),i=1,Nx1)
         write (io1,*) ' c='
         write (io1,*) (war(npwc+i),i=1,Nx1*Nx1)
         write (io1,*) ' b='
         write (io1,*) (war(nxb+i),i=1,mtt)
         write (io1,*) ' a='
         write (io1,*) (war(nbdx+i),i=1,Nx1*mtt)
         write (io1,*) ' xl='
         write (io1,*) (war(nxl+i),i=1,Nx1)
         write (io1,*) ' xu='
         write (io1,*) (war(nxu+i),i=1,Nx1)
      endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      call ql0001 (mtt,me,mmax,Nx1,nmax,mnn,
     1             war(npwc+1),war(npwd+1),war(nbdx+1),
     2             war(nxb+1),war(nxl+1),war(nxu+1),
     3             war(nxxl+1),war(nxdl+1),io1,ifail,
     4             iprnt,war(npww+1),lprr,iwar(mpww+1),lpir)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' After LP(x)'
         write (io1,*) ' lx='
         write (io1,*) (war(nxxl+i),i=1,Nx1)
         write (io1,*) ' dx='
         write (io1,*) (war(nxdl+i),i=1,mtt+2*Nx1)
      endif
      itilx=iabs(itrpw)
      isumpl=isumpl+itilx
      if (ifail.gt.2) then
         ipout(1)=ifail+300
         ipout(2)=0
         txt=' LP(x) : Failure'
         call msg (ipin,-1,1,txt,0,Zero)
         goto 9999
      endif
      Fmod=war(nxxl+Nx0)
      wwrt=Fmod
C-----
  220 txt='...LP_x: model'
      call msg (ipin,1,1,txt,0,wwrt)
      txt='         steps'
      call msg (ipin,0,1,txt,itilx,Zero)
C=====Checking whether minimum has been varied (y)=============
  230 continue
      itily=0
      txt='...LP_y: model'
      call msg (ipin,1,1,txt,0,Fmody)
      txt='         steps'
      call msg (ipin,0,1,txt,itily,Zero)
C-----
      itrc=itrc+1
      ici=itrc-icrit*(itrc/icrit)
      if (ici.ne.0) goto 245
C=====Computing minimum of the auxiliary problem===============
  235 continue
      ncrit=ncrit+1
      nxb=nxbt
  236 if (Nx.ne.0) then
         do i=1,Nx
            war(npwd+i)=-epsd2*war(nxxc+i)
         enddo
      endif
      war(npwd+Nx0)=Zero
      ww1=One
      if (indic0.ge.0) ww1=One-epsd2*Fmodx
      war(npwd+Nx1)=ww1
      nt=npwc
      do i=1,Nx1
         do j=1,Nx1
            nt=nt+1
            ww=Zero
            if (i.eq.j) ww=epsd
            war(nt)=ww
         enddo
      enddo
      mtt=Mtx+m
      me=mex
      mmax=mtt
      nmax=Nx1
      mnn=mtt+2*Nx1
      war(nxl+Nx0)=Zero
      war(nxu+Nx0)=Zero
      ww1=Fmodx
      war(nxl+Nx1)=ww1
      war(nxu+Nx1)=dinf
      iwar(mpww+1)=0
      itilc=0
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' Before Criterion '
         write (io1,*) ' d='
         write (io1,*) (war(npwd+i),i=1,Nx1)
         write (io1,*) ' c='
         write (io1,*) (war(npwc+i),i=1,Nx1*Nx1)
         write (io1,*) ' b='
         write (io1,*) (war(nxb+i),i=1,mtt)
         write (io1,*) ' a='
         write (io1,*) (war(nbdx+i),i=1,Nx1*mtt)
         write (io1,*) ' xl='
         write (io1,*) (war(nxl+i),i=1,Nx1)
         write (io1,*) ' xu='
         write (io1,*) (war(nxu+i),i=1,Nx1)
      endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      call ql0001 (mtt,me,mmax,Nx1,nmax,mnn,
     1             war(npwc+1),war(npwd+1),war(nbdx+1),
     2             war(nxb+1),war(nxl+1),war(nxu+1),
     3             war(nxxq+1),war(npwu+1),io1,ifail,
     4             iprnt,war(npww+1),lprr,iwar(mpww+1),lpir)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' After Criterion '
         write (io1,*) ' lx='
         write (io1,*) (war(nxxq+i),i=1,Nx1)
         write (io1,*) ' dx='
         write (io1,*) (war(nxdl+i),i=1,mtt+2*Nx1)
      endif
      itilc=iabs(itrpw)
      isumpc=isumpc+itilc
      if (ifail.gt.2) then
         ipout(1)=ifail+900
         ipout(2)=0
         txt=' LP(x) : Failure'
         call msg (ipin,-1,1,txt,0,Zero)
         goto 9999
      endif
      Fmodx=war(nxxq+Nx1)
      txt='...LP_c: model'
      call msg (ipin,1,1,txt,0,Fmodx)
      txt='         steps'
      call msg (ipin,0,1,txt,itilc,Zero)
C=====Computing level==========================================
  245 continue
      Diam=Fmax-Fmin
      Gap=Fmod
      Fmodc=dabs(Fmodx+Fmody)
      ff=Half*(dabs(Fmodx)+dabs(Fmody))
      if (indic.eq.0.and.Fmodx.ne.-dinf) then
         Fmodd=Fnorm*Diamx
         if (Fmodc.gt.Fmodd.and.Fmodd.ne.Zero) Fmodc=Fmodd
      endif
      Relacc=Fmodc/(One+ff)
      if (itr.eq.1) then
         Gapo=Fmod
         Gapi=Gapo
      endif
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' Relacc,Relshr,Gap,Gapo,Fmodc,m,lim=',
     *                       Relacc,Relshr,Gap,Gapo,Fmodc,m,lim
      endif
C=====If relative accuracy is OK===============================
 6666 continue
      if (Relacc.le.epsrel) then
         ipout(1)=0
         ipout(2)=1
         ipout(3)=itr
         itrp=itr
         if (imody.eq.0) then
            do j=1,Nx
               jj=ntrx+j
               xout(j)=war(jj+j)
            enddo
            call fun (indic,war(ntrx+1),Fxy,war(nbdxc+1),
     *                izs,rzs,dzs)
         else
            do j=1,Nx
               jj=ntrx+j
               xout(j)=war(jj+j)
            enddo
         endif
         goto 9998
      endif
C=====Auxiliary print after ifreq iterations===================
      if (ifreq.ne.0) then
         ii=itr-ifreq*(itr/ifreq)
         if (ii.eq.0.and.itrp.ne.itr) then
            ipout(1)=0
            ipout(2)=2
            ipout(3)=itr
            itrp=itr
            goto 9998
         endif
      endif
C====Shrinking bundle==========================================
      if (ishrb.le.0) goto 500
      if (Gap.gt.cfrelb*Gapo.and.m.lt.lim) goto 500
      if (m.eq.lim) ilim=ilim+1
      irl=0
      if (Relacc.eq.Two.or.Relacc.eq.dgr) irl=1
      if (Relacc.eq.Relshr.and.Gap.eq.Gapsh.and.irl.eq.0) then
         ipout(1)=-3
         ipout(2)=-2
         ipout(3)=itr
         goto 9999
      endif
C-----
      ixs=nbdx+Nx1*Mtx
      m1=0
      do i=1,m
         wx=war(nxdl+Mtx+i)
         if (wx.gt.Zero.or.i.eq.m) then
            ixt=nbdx+Nx1*(Mtx+m1)
            m1=m1+1
            do j=1,Nx1
               war(ixt+j)=war(ixs+j)
            enddo
            war(nxbt+Mtx+m1)=war(nxbt+Mtx+i)
            war(nxbs+Mtx+m1)=war(nxbs+Mtx+i)
            war(nxdl+Mtx+m1)=war(nxdl+Mtx+i)
         endif
         ixs=ixs+Nx1
      enddo
C-----
      m=m1
      Nmbndl=Nmbndl+1
      nbdxc=nbdx+(Mtx+m)*Nx1
      txt='...Shrinking bundle. Cardinality set to'
      call msg (ipin,0,1,txt,m,Zero)
      Gapo=Gap
      Relshr=Relacc
      Gapsh=Gap
C=====Solve QP problem=========================================
  500 continue
      alv=qlv*Fmod
      war(nxxc+Nx0)=alv
      ww1=Zero
      if (indic0.ge.0) ww1=-Fmody
      war(nxxc+Nx1)=ww1
      nt=npwc
      do i=1,Nx1
         war(npwd+i)=-war(nxxc+i)
         do j=1,Nx1
            nt=nt+1
            ww=Zero
            if (i.eq.j) ww=One
            war(nt)=ww
         enddo
      enddo
      mtt=Mtx+m
      me=mex
      mmax=mtt
      nmax=Nx1
      mnn=mtt+2*Nx1
      ww0=alv
      war(nxl+Nx0)=ww0
      war(nxu+Nx0)=ww0
      ww1=Zero
      if (indic0.ge.0) ww1=-Fmody
      war(nxl+Nx1)=ww1
      war(nxu+Nx1)=ww1
      iwar(mpww+1)=0
      itiq=0
      nxb=nxbt
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' Before QP(x)'
         write (io1,*) ' d='
         write (io1,*) (war(npwd+i),i=1,Nx1)
         write (io1,*) ' c='
         write (io1,*) (war(npwc+i),i=1,Nx1*Nx1)
         write (io1,*) ' b='
         write (io1,*) (war(nxb+i),i=1,mtt)
         write (io1,*) ' a='
         write (io1,*) (war(nbdx+i),i=1,Nx1*mtt)
         write (io1,*) ' zl='
         write (io1,*) (war(nxl+i),i=1,Nx1)
         write (io1,*) ' zu='
         write (io1,*) (war(nxu+i),i=1,Nx1)
      endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      call ql0001 (mtt,me,mmax,Nx1,nmax,mnn,
     1             war(npwc+1),war(npwd+1),war(nbdx+1),
     2             war(nxb+1),war(nxl+1),war(nxu+1),
     3             war(nxxq+1),war(npwu+1),io1,ifail,
     4             iprnt,war(npww+1),lprr,iwar(mpww+1),lpir)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if (idebl.ne.0) then
         write (io1,*)
         write (io1,*) itr,' After QP(x)'
         write (io1,*) ' lz='
         write (io1,*) (war(nxxq+i),i=1,Nx1)
         write (io1,*) ' dz='
         write (io1,*) (war(npwu+i),i=1,mtt+2*Nx1)
      endif
      itiq=iabs(itrpw)
      isumpq=isumpq+itiq
      if (ifail.gt.2) then
         ipout(1)=ifail+700
         ipout(2)=0
         txt=' QP : Failure'
         call msg (ipin,-1,1,txt,0,Zero)
         goto 9999
      endif
      txt='   QP: steps'
      call msg (ipin,0,1,txt,itiq,Zero)
      do i=1,Nx1
         war(nxxc+i)=war(nxxq+i)
      enddo
C=====END OF MAIN LOOP=========================================
 1000 Mav=Mav+m
      if (m.gt.Mxbndl) Mxbndl=m
      txt='Gap:     <='
      call msg (ipin,1,1,txt,0,Gap)
      goto 100
C=====ANSWER===================================================
 9999 continue
C-----
 9998 continue
      jo1=ipout(1)
      jo2=ipout(2)
      if (jo1.ne.0) then
         icodi=1000+jo1
         if (jo1.eq.-5) icodi=icodi+1000
         if (jo1.eq.-6) icodi=icodi+2000
      else
         icodi=jo2-1
      endif
      ipout(3)=itr
      ipout(4)=indic0+1
      ipout(5)=icodi
      ipout(6)=Mxbndl
      ipout(7)=Nmbndl
      ipout(8)=ncrit
      ipout(9)=isumor
      ipout(10)=isumpl
      ipout(11)=isumpq
      ipout(12)=isumpc
C-----
      Gap=Fmod
      Diam=Fmax-Fmin
      if (itr.ge.1) Avit=dfloat(Mav)/dfloat(itr)
      rpout(1)=Gap
      rpout(2)=-Fmodx
      rpout(3)=Fmody
      rpout(4)=Relacc
      rpout(5)=-Fmin
      rpout(6)=-Fmax
      rpout(7)=-Fxy
      rpout(8)=Avit
C-----
      if (jo1.eq.0.and.jo2.eq.2) goto 6666
C-----
 5555 return
      end
C--------------------------------------------------------------
      subroutine msg (ipin,jnt,iend,txt,nmb,rnmb)
      implicit real *8 (a-h,o-z)
      implicit integer *4 (i-n)
      dimension ipin(*)
      character *40 txt
C--------------------------------------------------------------
C     msg writes through channel ipin(3) the text "txt" and
C     then integer "nmb" (if jnt=0), real "rnmb" (if jnt>0)
C     or nothing (if jnt<0).
C     if ipin(2)=0 then the same is printed on the screen,
C     provided that iend=0, and if ipin(2)>0, then the output
C     onto the screen does not depend on iend.
C--------------------------------------------------------------
      i2=ipin(2)
      iout=ipin(3)
      if ((i2.eq.0.and.iend.eq.0).or.(i2.gt.0).or.
     *                    (i2.eq.-2.and.iend.eq.0)) then
         if (jnt.eq.0) write (*,*) txt,nmb
         if (jnt.gt.0) write (*,*) txt,rnmb
         if (jnt.lt.0) write (*,*) txt
      endif
      if ((i2.gt.-2.or.iend.eq.0).and.(i2.ne.-3)) then
         if (jnt.eq.0) write (iout,*) txt,nmb
         if (jnt.gt.0) write (iout,*) txt,rnmb
         if (jnt.lt.0) write (iout,*) txt
      endif
      return
      end
